
// This BSP compiler takes the polygon that splits other 
// polygons the least as the root polygon -- Xavier


#include "stdafx.h"
#include <conio.h>

#define PERPDIST(V,P) ((V.x - verts[P->ind[0]].x) * P->nx +		\
					   (V.y - verts[P->ind[0]].y) * P->ny +		\
					   (V.z - verts[P->ind[0]].z) * P->nz )

#define SQUAREDIST(V1,V2) ((V1.x - V2.x)*(V1.x - V2.x) +		\
						   (V1.y - V2.y)*(V1.y - V2.y) +		\
						   (V1.z - V2.z)*(V1.z - V2.z) )

#define APPROXEQUAL(A,B) (((A) > (B) - 0.05f) &&	\
						  ((A) < (B) + 0.05f))

struct VERTEX
{
  float x,y,z;
  float tu,tv;
};

struct POLYGON
{
  float midx,midy,midz;
  float nx,ny,nz;
  unsigned short numind;
  unsigned short ind[50];

  POLYGON *next,*front,*back;
  unsigned short Number;
  unsigned short TextureIndex;
};

struct LIGHT
{
	float x1,y1,z1;
	float x2,y2,z2;
	float Radius;
	float Brightness;
	float R,G,B;
	unsigned short LightType;
};

struct WATER
{
	float y;
	float R,G,B;
	unsigned short NumBoundingPolys;
	unsigned short BoundingPolyInds[50];
};



unsigned short	numverts;
VERTEX			verts[50000];

unsigned short	numpolys;
POLYGON	polys[50000];
unsigned short	BSPPolyNum=0;
unsigned short	BSPPolyNum0=0;

unsigned short	numlights;
LIGHT	lights[1000];

unsigned short	numwaters;
WATER	waters[1000];


FILE *infile,*outfile;


unsigned short	NumTextures;
struct	TEXTUREPROPERTY
{
	char	TextureFileName[100];
	char	HasAlpha,IsGlass,HitChecked;
};
TEXTUREPROPERTY Textures[50];


unsigned short	UnsignedShortDummy;


float			Scale;


// Function declarations

void CalcPolyMidPt(POLYGON *tpoly);
int CalcPolyNormal(POLYGON *tpoly);
void BuildBSPTree(POLYGON *polyhead, POLYGON **ParentsChildPointer);
void WritePolyDataBIN(POLYGON *tpoly);
void WritePolyDataRM(POLYGON *tpoly);
void CountPolys(POLYGON *tpoly);


// The starting point
int main(int argc, char* argv[])
{	
	POLYGON *BSPHead;
	char tfilename[50];

	if(argc < 3)
	{
		printf("Insufficient arguments\n");
		printf("Syntax : BSP2 <RMD-filename> <scale>\n");
		getch();
		return 0;
	}

	sscanf(argv[2],"%f",&Scale);


	strcpy(tfilename,argv[1]);
	strcat(tfilename,".RMD");

	if((infile = fopen(tfilename,"r")) == (FILE *)NULL)
	{
		printf("Fatal Error : Cannot open input-file\n");
		getch();
		return 0;
	}

	unsigned int dui;

	// Read the object data
	printf("Reading data... ");
	
	// Read the vertex data
	fscanf(infile,"%u",&dui);
	numverts = dui;

	for(int i=0;i<numverts;i++)
	{
		fscanf(infile,"%f,%f,%f",&verts[i].x, &verts[i].y, &verts[i].z);
		fscanf(infile,"%f,%f",&verts[i].tu, &verts[i].tv);
	}

	// Read the polygon data
	fscanf(infile,"%u",&dui);
	numpolys = dui;
	for(i=0;i<numpolys;i++)
	{
		fscanf(infile,"%u",&dui);
		polys[i].numind = dui;
		for(int j=0;j<polys[i].numind;j++)
		{
			fscanf(infile,"%u",&dui);
			polys[i].ind[j] = dui;
		}
		fscanf(infile,"%u",&dui);
		polys[i].TextureIndex = dui;
	}

	fscanf(infile,"%u\n",&dui);
	NumTextures = dui;
	int   dummy1;
	float dummy2;
	for(i=0;i<NumTextures;i++)
	{
		dummy1 = fgetc(infile);
		fgets(Textures[i].TextureFileName,100,infile);
		Textures[i].TextureFileName[strlen(Textures[i].TextureFileName)-2] = '\0';

		fscanf(infile,"%u,",&dummy1);
		if(dummy1)
			Textures[i].HasAlpha = 1;
		else
			Textures[i].HasAlpha = 0;

		fscanf(infile,"%u,",&dummy1);
		if(dummy1)
			Textures[i].IsGlass  = 1;
		else
			Textures[i].IsGlass  = 0;

		fscanf(infile,"%f,%d,%f,%f,%u,",&dummy2,&dummy1,&dummy2,&dummy2,&dummy1);

		fscanf(infile,"%u\n",&dummy1);
		if(dummy1)
			Textures[i].HitChecked = 1;
		else
			Textures[i].HitChecked = 0;
	}


	fscanf(infile,"%u",&dui);
	numlights = dui;
	for(i=0;i<numlights;i++)
	{
		fscanf(infile,"%f,%f,%f",&lights[i].x1, &lights[i].y1, &lights[i].z1);
		fscanf(infile,"%f,%f,%f",&lights[i].x2, &lights[i].y2, &lights[i].z2);
		fscanf(infile,"%f",&lights[i].Radius);	// Dummy used to read the smaller radius
		fscanf(infile,"%f",&lights[i].Radius);
		fscanf(infile,"%f",&lights[i].Brightness);
		fscanf(infile,"%f,%f,%f",&lights[i].R, &lights[i].G, &lights[i].B);
		fscanf(infile,"%u",&dui);
		lights[i].LightType = dui;
	}
	
	fscanf(infile,"%u",&dui);
	numwaters = dui;
	for(i=0;i<numwaters;i++)
	{
		fscanf(infile,"%f",&waters[i].y);
		fscanf(infile,"%f,%f,%f",&waters[i].R, &waters[i].G, &waters[i].B);
		fscanf(infile,"%u",&dui);
		waters[i].NumBoundingPolys = dui;
		for(int j=0;j<waters[i].NumBoundingPolys;j++)
		{
			fscanf(infile,"%u",&dui);
			waters[i].BoundingPolyInds[j] = dui;
		}
	}


	fclose(infile);
	printf("Complete\n");


	// Calculate some necessary data
	printf("Calculating necessary data... ");

	// Find the mid-pt of all the vertices
	float cx,cy,cz;
	cx=cy=cz=0.0f;
	
	for(i=0;i<numverts;i++)
	{	cx += verts[i].x;
		cy += verts[i].y;
		cz += verts[i].z;
	}
	
	cx /= (float)numverts;
	cy /= (float)numverts;
	cz /= (float)numverts;
	
	// Centerize and scale all the vertices
	for(i=0;i<numverts;i++)
	{
		verts[i].x = (verts[i].x - cx) * Scale;
		verts[i].y = (verts[i].y - cy) * Scale;
		verts[i].z = (verts[i].z - cz) * Scale;

		// Scale the Tex coords also
		verts[i].tu = verts[i].tu * Scale;
		verts[i].tv = verts[i].tv * Scale;
	}
	
	// Centerize and scale all the lights
	for(i=0;i<numlights;i++)
	{
		lights[i].x1 = (lights[i].x1 - cx) * Scale;
		lights[i].y1 = (lights[i].y1 - cy) * Scale;
		lights[i].z1 = (lights[i].z1 - cz) * Scale;

		if(lights[i].x2 || lights[i].y2 || lights[i].z2)
		{
			lights[i].x2 = (lights[i].x2 - cx) * Scale;
			lights[i].y2 = (lights[i].y2 - cy) * Scale;
			lights[i].z2 = (lights[i].z2 - cz) * Scale;
		}

		lights[i].Radius *= Scale;
	}

	// Centerize and scale all the waters
	for(i=0;i<numwaters;i++)
		waters[i].y = (waters[i].y - cy) * Scale;


	for(i=0;i<numpolys;i++)
	{	
		// Set the next pointer
		polys[i].next = &polys[i+1];

		// Calculate the mid point of the polygon
		CalcPolyMidPt(&polys[i]);
		
		// Calculate the normal of the polygon
		if(!CalcPolyNormal(&polys[i]))
		{	printf("Stopped\n");
			printf("Error : An invalid polygon was found at index %d\n",i);
			getch();
		  	return 0;
		}
	}
	// Set the next pointer
	polys[i-1].next = NULL;

	printf("Complete\n");


	// Build the BSP Tree
	printf("Building BSP Tree... ");
	BuildBSPTree(polys,&BSPHead);
	printf("Complete\n");


	// Assign the Numbers to each Polygon
	CountPolys(BSPHead);


	//Correct the indices of the water polys
	for(i=0;i<numwaters;i++)
	 for(int j=0;j<waters[i].NumBoundingPolys;j++)
	  waters[i].BoundingPolyInds[j] = polys[waters[i].BoundingPolyInds[j]].Number;



	// Write the Binary Data

	strcpy(tfilename,"Rdata.bin");

	if((outfile = fopen(tfilename,"wb")) == (FILE *)NULL)
	{
		printf("Fatal Error : Cannot create output-file : %s\n",tfilename);
		getch();
		return 0;
	}

	printf("Writing Binary Room data... ");

	// Write the no. of vertices
	fwrite(&numverts,sizeof(numverts),1,outfile);
	// Write the vertices
	for(i=0;i<numverts;i++)
	{
		fwrite(&(verts[i].x),sizeof(float),1,outfile);
		fwrite(&(verts[i].y),sizeof(float),1,outfile);
		fwrite(&(verts[i].z),sizeof(float),1,outfile);
	}
	for(i=0;i<numverts;i++)
	{
		fwrite(&(verts[i].tu),sizeof(float),1,outfile);
		fwrite(&(verts[i].tv),sizeof(float),1,outfile);
	}


	// Write the no. of BSP polygons
	fwrite(&numpolys,sizeof(numpolys),1,outfile);
	// Write the polygon data
	WritePolyDataBIN(BSPHead);
	// Write the no. of components
	fwrite(&NumTextures,sizeof(NumTextures),1,outfile);
	// Write the component Data
	for(i=1;i<=NumTextures;i++)
	{
		fwrite(&(Textures[i-1].HasAlpha),sizeof(char),1,outfile);
		fwrite(&(Textures[i-1].IsGlass ),sizeof(char),1,outfile);
		fwrite(&(Textures[i-1].HitChecked),sizeof(char),1,outfile);


		UnsignedShortDummy = 0;
		for(int j=0;j<numpolys;j++)
		 if(polys[j].TextureIndex == i)
			 UnsignedShortDummy++;

		fwrite(&UnsignedShortDummy,sizeof(unsigned short),1,outfile);
		for(j=0;j<numpolys;j++)
		 if(polys[j].TextureIndex == i)
			 fwrite(&(polys[j].Number),sizeof(unsigned short),1,outfile);
	}
	
	// Write the Texture File names
	for(i=0;i<NumTextures;i++)
	{	UnsignedShortDummy = strlen(Textures[i].TextureFileName);
		fwrite(&UnsignedShortDummy,sizeof(unsigned short),1,outfile);
		fwrite(Textures[i].TextureFileName,sizeof(char),UnsignedShortDummy,outfile);
	}

	// Count the no. of non-ambient lights
	unsigned short numnonambientlights=0;
	for(i=0;i<numlights;i++)
		if(lights[i].LightType == 2 ||
		   lights[i].LightType == 3 )
			numnonambientlights++;
	// Write the no. of non-ambient lights
	fwrite(&numnonambientlights,sizeof(unsigned short),1,outfile);
	// Write the non-ambient lights
	for(i=0;i<numlights;i++)
		if(lights[i].LightType == 2 ||
		   lights[i].LightType == 3 )
			fwrite(&lights[i],sizeof(float)*11,1,outfile);
	
	// Write the no. of waters
	fwrite(&numwaters,sizeof(unsigned short),1,outfile);
	// Write the waters
	for(i=0;i<numwaters;i++)
	{
		fwrite(&waters[i].y,sizeof(float),1,outfile);
		fwrite(&waters[i].R,sizeof(float),1,outfile);
		fwrite(&waters[i].G,sizeof(float),1,outfile);
		fwrite(&waters[i].B,sizeof(float),1,outfile);
		fwrite(&waters[i].NumBoundingPolys,sizeof(unsigned short),1,outfile);
		for(int j=0;j<waters[i].NumBoundingPolys;j++)
			fwrite(&waters[i].BoundingPolyInds[j],sizeof(unsigned short),1,outfile);
	}


	fclose(outfile);
	printf("Complete\n");


	return 0;
}


void CalcPolyMidPt(POLYGON *tpoly)
{
	tpoly->midx = tpoly->midy = tpoly->midz = 0.0f;

	for(int i=0;i<tpoly->numind;i++)
	{
		tpoly->midx += verts[tpoly->ind[i]].x;
		tpoly->midy += verts[tpoly->ind[i]].y;
		tpoly->midz += verts[tpoly->ind[i]].z;
	}

	tpoly->midx /= i;
	tpoly->midy /= i;
	tpoly->midz /= i;
}


int CalcPolyNormal(POLYGON *tpoly)
{
	float tx1,ty1,tz1,tx2,ty2,tz2,Denom;

	tx1 = verts[tpoly->ind[0]].x - verts[tpoly->ind[1]].x;
	ty1 = verts[tpoly->ind[0]].y - verts[tpoly->ind[1]].y;
	tz1 = verts[tpoly->ind[0]].z - verts[tpoly->ind[1]].z;

	tx2 = verts[tpoly->ind[2]].x - verts[tpoly->ind[1]].x;
	ty2 = verts[tpoly->ind[2]].y - verts[tpoly->ind[1]].y;
	tz2 = verts[tpoly->ind[2]].z - verts[tpoly->ind[1]].z;

	
	tpoly->nx = ty2 * tz1 - ty1 * tz2;
	tpoly->ny = tx1 * tz2 - tx2 * tz1;
	tpoly->nz = tx2 * ty1 - tx1 * ty2;

	Denom = sqrtf(tpoly->nx * tpoly->nx + tpoly->ny * tpoly->ny + tpoly->nz * tpoly->nz);

	if(Denom == 0.0f)
		return 0;

	tpoly->nx /= Denom;
	tpoly->ny /= Denom;
	tpoly->nz /= Denom;

	return 1;
}



POLYGON *ppoly,*pnextpoly,*pspoly;
int i,j,count,frontpolythere,backpolythere;
float Rcx,Rcy,Rcz;
float tx,ty,tz,t,closestDist;
int numfrontind,numbackind;
int frontind[50],backind[50];
float prevdist,currdist;
int notfoundflag=1;


void BuildBSPTree(POLYGON *polyhead, POLYGON **ParentsChildPointer)
{
	POLYGON *pRootPoly,*pFrontPolys,*pBackPolys;


//////////////////////////////////////////////////////////////////
//		Code to take the polygon that splits other polygons		//
//		the least as the Root									//
//////////////////////////////////////////////////////////////////

	count = 0;
	pspoly = polyhead->next;
	while(pspoly)
	{
		frontpolythere = backpolythere = 0;
		for(j=0;j < pspoly->numind;j++)
		{
			t = PERPDIST(verts[pspoly->ind[j]],polyhead);

			if(t > 0.05f)
			{ frontpolythere = 1; if(backpolythere) { count++; break; }}
			else if(t < -0.05f)
			{ backpolythere = 1;  if(frontpolythere){ count++; break; }}
		}

		pspoly = pspoly->next;
	}
	pRootPoly = polyhead;

	ppoly = polyhead->next;
	while(ppoly)
	{
		i=0;
		pspoly = polyhead;
		while(pspoly)
		{
			if(pspoly != ppoly)
			{
				frontpolythere = backpolythere = 0;
				for(j=0;j < pspoly->numind;j++)
				{
					t = PERPDIST(verts[pspoly->ind[j]],ppoly);

					if(t > 0.05f)
					{ frontpolythere = 1; if(backpolythere) { i++; break; }}
					else if(t < -0.05f)
					{ backpolythere = 1;  if(frontpolythere){ i++; break; }}
				}
			}
			pspoly = pspoly->next;
		}

		if(i < count)
		{
			count = i; pRootPoly = ppoly;
		}
		ppoly = ppoly->next;
	}

////////////////////////////////////////////////////////////////


/*
//////////////////////////////////////////////////////////////
//		Code to take the middle-most polygon as the Root	//
//////////////////////////////////////////////////////////////

	// Find the center of all the polygons

	count=0;
	Rcx = Rcy = Rcz = 0.0f;

	ppoly = polyhead;
	while(ppoly)
	{
		Rcx += ppoly->midx;
		Rcy += ppoly->midy;
		Rcz += ppoly->midz;

		count++;
		ppoly = ppoly->next;
	}
	Rcx /= count; Rcy /= count; Rcz/=count;


	// Find which polygon is closest to the center
	
	tx = polyhead->midx - Rcx;
	ty = polyhead->midy - Rcy;
	tz = polyhead->midz - Rcz;
	closestDist = tx*tx + ty*ty + tz*tz;
	pRootPoly = polyhead;

	ppoly = polyhead->next;
	while(ppoly)
	{
		tx = ppoly->midx - Rcx;
		ty = ppoly->midy - Rcy;
		tz = ppoly->midz - Rcz;

		t = tx*tx + ty*ty + tz*tz;
		if(t < closestDist)
		{ closestDist = t; pRootPoly = ppoly; }
	
		ppoly = ppoly->next;
	}

///////////////////////////////////////////////////////////////
*/


	*ParentsChildPointer = pRootPoly;
	pFrontPolys = pBackPolys = NULL;


	// Find the polygons which are at the front/back
	// of 'pRootPoly' (split polygons if necessary)
	// and push them into 'pFrontPolys' & 'pBackPolys'

	ppoly = polyhead;
	while(ppoly)
	{
		if(ppoly == pRootPoly)
		{
			ppoly = ppoly->next; // Skip the root polygon
		}
		else
		{
			pnextpoly = ppoly->next;

			// See if there are polygons in front/back/front&back
			frontpolythere = backpolythere = 0;
			for(i=0;i < ppoly->numind;i++)
			{
				t = PERPDIST(verts[ppoly->ind[i]],pRootPoly);

				if(t > 0.05f)
				{ frontpolythere = 1; if(backpolythere) break; }
				else if(t < -0.05f)
				{ backpolythere = 1;  if(frontpolythere) break; }
			}

			if(frontpolythere && backpolythere)
			{
				// We'll need an extra polygon( a polygon got split)
				pspoly = &polys[numpolys++];

				// The normal remains the same
				pspoly->nx = ppoly->nx;
				pspoly->ny = ppoly->ny;
				pspoly->nz = ppoly->nz;
				pspoly->TextureIndex = ppoly->TextureIndex;
				pspoly->Number = numpolys;


				prevdist = 0.0f; numfrontind = numbackind = 0;
				for(i=0;i < ppoly->numind;i++)
				{
					currdist = PERPDIST(verts[ppoly->ind[i]],pRootPoly);

					if(prevdist * currdist < 0.0f)
					{
						t = prevdist/(prevdist - currdist);
						tx = verts[ppoly->ind[i-1]].x + (verts[ppoly->ind[i]].x - verts[ppoly->ind[i-1]].x)*t;
						ty = verts[ppoly->ind[i-1]].y + (verts[ppoly->ind[i]].y - verts[ppoly->ind[i-1]].y)*t;
						tz = verts[ppoly->ind[i-1]].z + (verts[ppoly->ind[i]].z - verts[ppoly->ind[i-1]].z)*t;

						verts[numverts].x=tx;
						verts[numverts].y=ty;
						verts[numverts].z=tz;
							
						verts[numverts].tu = 
							verts[ppoly->ind[i-1]].tu + (verts[ppoly->ind[i]].tu - verts[ppoly->ind[i-1]].tu)*t;
						verts[numverts].tv = 
							verts[ppoly->ind[i-1]].tv + (verts[ppoly->ind[i]].tv - verts[ppoly->ind[i-1]].tv)*t;

						frontind[numfrontind++]	= numverts;
						backind[numbackind++]	= numverts;
						numverts++;
					}

					if(currdist > 0.0f)
						frontind[numfrontind++] = ppoly->ind[i];
					else if(currdist < 0.0f)
						backind[numbackind++] = ppoly->ind[i];
					else
					{
						frontind[numfrontind++] = ppoly->ind[i];
						backind[numbackind++] = ppoly->ind[i];
					}

					prevdist = currdist;
				}

		
				// Special case for the first and last vertices

				currdist = PERPDIST(verts[ppoly->ind[0]],pRootPoly);
				
				if(prevdist * currdist < 0.0f)
				{
					t = prevdist/(prevdist - currdist);
					tx = verts[ppoly->ind[i-1]].x + (verts[ppoly->ind[0]].x - verts[ppoly->ind[i-1]].x)*t;
					ty = verts[ppoly->ind[i-1]].y + (verts[ppoly->ind[0]].y - verts[ppoly->ind[i-1]].y)*t;
					tz = verts[ppoly->ind[i-1]].z + (verts[ppoly->ind[0]].z - verts[ppoly->ind[i-1]].z)*t;

					verts[numverts].x=tx;
					verts[numverts].y=ty;
					verts[numverts].z=tz;

					verts[numverts].tu = 
						verts[ppoly->ind[i-1]].tu + (verts[ppoly->ind[0]].tu - verts[ppoly->ind[i-1]].tu)*t;
					verts[numverts].tv = 
						verts[ppoly->ind[i-1]].tv + (verts[ppoly->ind[0]].tv - verts[ppoly->ind[i-1]].tv)*t;

					frontind[numfrontind++]	= numverts;
					backind[numbackind++]	= numverts;
					numverts++;
				}


				// Remove unnecessary vertices from 
				// the front polygon
				i=1;
				while(i<numfrontind)
				{
					if( APPROXEQUAL(verts[frontind[i]].x,verts[frontind[i-1]].x) &&
						APPROXEQUAL(verts[frontind[i]].y,verts[frontind[i-1]].y) &&
						APPROXEQUAL(verts[frontind[i]].z,verts[frontind[i-1]].z) )
					{
						// Remove this vertex
						numfrontind--;
						for(j=i;j<numfrontind;j++)
							frontind[j] = frontind[j+1];
					}
					else
						i++;
				}
				if(numfrontind > 3)
				if( APPROXEQUAL(verts[frontind[numfrontind-1]].x,verts[frontind[0]].x) &&
					APPROXEQUAL(verts[frontind[numfrontind-1]].y,verts[frontind[0]].y) &&
					APPROXEQUAL(verts[frontind[numfrontind-1]].z,verts[frontind[0]].z) )
						numfrontind--;
				

				// Remove unnecessary vertices from 
				// the back polygon
				i=1;
				while(i<numbackind)
				{
					if( APPROXEQUAL(verts[backind[i]].x,verts[backind[i-1]].x) &&
						APPROXEQUAL(verts[backind[i]].y,verts[backind[i-1]].y) &&
						APPROXEQUAL(verts[backind[i]].z,verts[backind[i-1]].z) )
					{
						// Remove this vertex
						numbackind--;
						for(j=i;j<numbackind;j++)
							backind[j] = backind[j+1];
					}
					else
						i++;
				}
				if(numbackind > 3)
				if( APPROXEQUAL(verts[backind[numbackind-1]].x,verts[backind[0]].x) &&
					APPROXEQUAL(verts[backind[numbackind-1]].y,verts[backind[0]].y) &&
					APPROXEQUAL(verts[backind[numbackind-1]].z,verts[backind[0]].z) )
						numbackind--;

				
				// Copy the front vertices into 'ppoly'
				if(numfrontind > 2)
				{
					ppoly->numind = numfrontind;
					for(i=0;i<numfrontind;i++)
						ppoly->ind[i] = frontind[i];
				
					CalcPolyMidPt(ppoly);

					// Push 'ppoly' into 'pFrontPolys'
					ppoly->next=pFrontPolys; pFrontPolys = ppoly;
				}

				// Copy the back vertices into 'pspoly'
				if(numbackind > 2)
				{
					pspoly->numind = numbackind;
					for(i=0;i<numbackind;i++)
						pspoly->ind[i] = backind[i];
				
					CalcPolyMidPt(pspoly);

					// Push 'pspoly' into 'pBackPolys'
					pspoly->next=pBackPolys; pBackPolys = pspoly;
				}
				else
					numpolys--;
			}
			else if(backpolythere)
			{
				// Push 'ppoly' into 'pBackPolys'
				ppoly->next=pBackPolys; pBackPolys = ppoly;
			}
			else
			{
				// Push 'ppoly' into 'pFrontPolys'
				ppoly->next=pFrontPolys; pFrontPolys = ppoly;
			}

			ppoly = pnextpoly;

		}
	}


	if(pFrontPolys == NULL)
		pRootPoly->front = NULL;
	else
		BuildBSPTree(pFrontPolys,&(pRootPoly->front));


	if(pBackPolys == NULL)
		pRootPoly->back = NULL;
	else
		BuildBSPTree(pBackPolys,&(pRootPoly->back));

}


void WritePolyDataBIN(POLYGON *tpoly)
{
	fwrite(&(tpoly->numind),sizeof(unsigned short),1,outfile);
	for(int j=0;j<tpoly->numind;j++)
		fwrite(&(tpoly->ind[j]),sizeof(unsigned short),1,outfile);
	
	if(tpoly->front)
		fwrite(&((tpoly->front)->Number),sizeof(unsigned short),1,outfile);
	else
		fwrite(&BSPPolyNum0,sizeof(unsigned short),1,outfile);

	if(tpoly->back)
		fwrite(&((tpoly->back)->Number),sizeof(unsigned short),1,outfile);
	else
		fwrite(&BSPPolyNum0,sizeof(unsigned short),1,outfile);


	if(tpoly->front)
		WritePolyDataBIN(tpoly->front);
	if(tpoly->back)
		WritePolyDataBIN(tpoly->back);
}


void WritePolyDataRM(POLYGON *tpoly)
{
	fprintf(outfile,"%d\n",tpoly->numind);
	for(int j=0;j<tpoly->numind;j++)
		fprintf(outfile,"%d\n",tpoly->ind[j]);
		
	fprintf(outfile,"%d\n",tpoly->TextureIndex);

	if(tpoly->front)
		WritePolyDataRM(tpoly->front);
	if(tpoly->back)
		WritePolyDataRM(tpoly->back);
}


void CountPolys(POLYGON *tpoly)
{
	tpoly->Number = BSPPolyNum++;

	if(tpoly->front)
		CountPolys(tpoly->front);
	if(tpoly->back)
		CountPolys(tpoly->back);
}	

